package com.mideaepay.demo.service;

import com.mideaepay.demo.model.resp.HttpResponse;
import com.mideaepay.demo.utils.HttpClientUtils;
import com.mideaepay.demo.utils.SignUtils;
import com.mideaepay.demo.utils.StringUtils;
import com.mideaepay.demo.utils.TradeUtils;
import com.mideaepay.demo.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;

/**
 * 特别声明：demo代码只是为了方便商户测试而提供的样例代码，商户可以根据自己网站的需要，按照技术文档编写，并非一定要使用该代码。该代码仅供参考！！！
 * @author harvey
 * @date 2020/9/2
 * @email husy7@midea.com
 */
public abstract class AbstractGateWayService implements GatewayService{

    /**
     * 自定义配置信息
     */
    protected Properties customProperties;

    /**
     * 公共配置信息
     */
    protected static Properties commonProperties;

    /**
     * 自定义配置文件路径
     */
    protected String customConfigPath;

    /**
     * 公共配置文件路径
     */
    protected static String commonConfigPath = "config/common_config.properties";

    /**
     * 有文件上传才使用
     */
    protected File file;

    static {
        //加载公共配置信息
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        commonProperties = new Properties(System.getProperties());

        try {
            commonProperties.load(new BufferedReader(
                new InputStreamReader(Objects.requireNonNull(cl.getResourceAsStream(commonConfigPath)),
                    StandardCharsets.UTF_8)));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    protected abstract void setCustomConfigPath();
    
    protected abstract void initBusinessRequest(Map<String, String> params, HttpServletRequest request);

    @Override
    public void service(HttpServletRequest request, HttpServletResponse resp) {

        //设置配置文件路径
        setCustomConfigPath();

        //加载配置文件
        loadCustomConfig();

        //初始化基本参数
        Map<String, String> params = initBaseRequest();

        //初始化业务参数
        initBusinessRequest(params, request);

        //签名
        doSign(params);
        
        if(isForm()){
          //不请求美的支付,返回form表单,让用户进行页面跳转
          String form = WebUtils.buildForm(commonProperties.getProperty("req.service.url"), params);
          WebUtils.writeHtml(resp, form);
        }else {
            //请求美的支付,返回json格式的数据
            String json = doService(params, file);
            //验证返回数据的签名
            SignUtils.checkSign(json);
            //输出响应数据到前端
            WebUtils.writeJson(resp, json);
        }

    }
    
    protected boolean isForm() {
        ServiceAnnotation annotation = this.getClass().getAnnotationsByType(ServiceAnnotation.class)[0];
        return annotation.httpForm();
    }
    
    protected void doSign(Map<String, String> params) {
        //签名
        String sign = SignUtils.getSign(SignUtils.getStringToSignOfStr(params));
        params.put("sign", sign);
    }

    protected String doService(Map<String, String> params, File file) {
        HttpResponse httpResp = null;
        if (file == null) {
            httpResp = HttpClientUtils.doPost(commonProperties.getProperty("req.service.url"), params);
        } else {
            httpResp = HttpClientUtils.doPostFile(commonProperties.getProperty("req.service.url"), params, file);
        }
        WebUtils.checkJson(httpResp);
        return httpResp.getEntity();
    }

    protected Map<String, String> initBaseRequest() {

        Map<String, String> params = new HashMap<>();
        ServiceAnnotation annotation = this.getClass().getAnnotationsByType(ServiceAnnotation.class)[0];
        params.put("service", annotation.service());
        params.put("partner", commonProperties.getProperty("partner"));
        params.put("version", commonProperties.getProperty("version"));
        params.put("input_charset", commonProperties.getProperty("input_charset"));
        params.put("language", commonProperties.getProperty("language"));
        params.put("sign_type", commonProperties.getProperty("sign_type"));
        params.put("req_seq_no", TradeUtils.getReqSeqNo());

        return params;
    }

    protected void loadCustomConfig() {
        if(StringUtils.isEmpty(customConfigPath)) {
            return;
        }
        //加载配置信息
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        customProperties = new Properties(System.getProperties());
        try {
            customProperties.load(new BufferedReader(
                new InputStreamReader(Objects.requireNonNull(cl.getResourceAsStream(customConfigPath)),
                    StandardCharsets.UTF_8)));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
