package com.mideaepay.demo.service;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import org.reflections.Reflections;

/**
 * 特别声明：demo代码只是为了方便商户测试而提供的样例代码，商户可以根据自己网站的需要，按照技术文档编写，并非一定要使用该代码。该代码仅供参考！！！
 * @desc <service工厂>
 * @author kanbo
 * @date 2020-9-1 12:20:12
 */
public class GatewayServiceFactory {
    
    private static GatewayServiceFactory instance = new GatewayServiceFactory();
    
    private Map<String, GatewayService> serviceMap = new HashMap<>();
    
    private GatewayServiceFactory() {
        // 入参 要扫描的包名
        Reflections f = new Reflections("com.mideaepay.demo.service.impl");
        // 入参 目标注解类
        Set<Class<?>> set = f.getTypesAnnotatedWith(ServiceAnnotation.class);
        
        for (Class<?> c : set) {
            Object bean = null;
            try {
                bean = c.newInstance();
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
            ServiceAnnotation annotation = c.getAnnotation(ServiceAnnotation.class);
            String annService = annotation.service();
            serviceMap.put(annService, (GatewayService) bean);
        }
    }
    
    public GatewayService produce(String service) {
        
        return serviceMap.get(service);
    }
    
    public static GatewayServiceFactory getInstance() {
        return instance;
    }
    
}
