package com.mideaepay.demo.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.mideaepay.demo.model.resp.CheckSignResponse;
import com.mideaepay.demo.model.resp.HttpResponse;
import com.mideaepay.demo.model.resp.SignResponse;
import com.mideaepay.demo.utils.Constant.Key;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * 特别声明：demo代码只是为了方便商户测试而提供的样例代码，商户可以根据自己网站的需要，按照技术文档编写，并非一定要使用该代码。该代码仅供参考！！！
 * 签名工具
 *
 * @author harvey
 * @date 2020/9/1
 * @email husy7@midea.com
 */
public class SignUtils{

    private static final String SUCC_CODE = "1";

    public static String partnerAesEncrypt(String text, String partner, String reqSeqNo, String key) {

        String md5Source = new StringBuilder().append("partner=").append(partner).append("&req_seq_no=")
            .append(reqSeqNo).append("&key=").append(key).toString();
        String aesKey = DigestUtils.getMD5String(md5Source).substring(8, 24).toLowerCase();

        return DigestUtils.aesEncrypt(text, aesKey);
    }

    public static String getSign(String sourceSign) {

        Map<String, String> params = new HashMap<>();
        params.put("source", DigestUtils.encryptMD5(sourceSign));
        //此处商户应该调用自己部署的tw-rsa-web程序
        HttpResponse httpResp = HttpClientUtils.doPost(
        		ConfigStatic.getConfigProperty("get.sign.url"), params);
        WebUtils.checkJson(httpResp);
        SignResponse signResponse = JSON.parseObject(httpResp.getEntity(), SignResponse.class);
        if (signResponse != null && !SUCC_CODE.equals(signResponse.getCode())) {
            throw new RuntimeException("get tw sign exception");
        }
        return signResponse.getSign();

    }

    public static void checkSign(String json) {

        Map<String, String> resultMap = JSON.parseObject(json, new TypeReference<Map<String, String>>(){

        });
        String sign = resultMap.remove(Key.SIGN);
        String sourceSign = getStringToSignOfStr(resultMap);
        Map<String, String> params = new HashMap<>();
        params.put("source", DigestUtils.encryptMD5(sourceSign));
        params.put("sign", sign);
        HttpResponse resp = HttpClientUtils.doPost(
        		ConfigStatic.getConfigProperty("check.sign.url"), params);
        WebUtils.checkJson(resp);
        CheckSignResponse checkSignResponse = JSON.parseObject(resp.getEntity(), CheckSignResponse.class);
        if (SUCC_CODE.equals(checkSignResponse.getCode())) {
            if (Constant.SUCC_RESULT_CODE.equals(resultMap.get(Key.RESULT_CODE))) {
                //验证签名成功&&返回的结果码为成功(如果是交易接口,还需要判断订单的状态),可做业务逻辑处理
                System.out.println("交易响应码为成功");
            } else {
                System.err.println("交易响应码异常:" + resultMap.get(Key.RESULT_CODE));
            }
        } else {
            //验证签名失败,不能做业务逻辑处理
            System.err.println("验证签名失败");
        }
    }

    public static String getStringToSignOfStr(Map<String, String> map) {

        TreeMap<String, String> treeMap = new TreeMap<>();
        Iterator<String> iterator = map.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            treeMap.put(key, map.get(key));
        }
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String, String> entry : treeMap.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            if (value == null || "".equals(value)) {
                continue;
            } else {
                sb.append(key).append("=").append(value).append("&");
            }
        }
        if (sb.length() > 1)
            return sb.deleteCharAt(sb.length() - 1).toString();
        else
            return "";
    }

}
